package com.flyfish.framework.domain.authorized;

import com.flyfish.framework.builder.CriteriaBuilder;
import com.flyfish.framework.domain.base.NameLikeQo;
import com.flyfish.framework.domain.po.Department;
import lombok.Getter;
import lombok.Setter;

import java.util.Collections;
import java.util.List;

/**
 * 带鉴权的查询实体，主要以部门隔绝
 *
 * @param <T>
 */
@Getter
@Setter
public abstract class AuthorizedQo<T extends AuthorizedDomain> extends NameLikeQo<T> {

    /**
     * 获取可见的权限ids
     *
     * @return 结果
     */
    public List<String> getAuthorizedIds() {
        if (user instanceof AuthorizedUserDetails) {
            return ((AuthorizedUserDetails) user).getAuthorityCodes();
        }
        return Collections.singletonList(Department.PUBLIC);
    }

    @Override
    public CriteriaBuilder<T> criteriaBuilder() {
        return super.criteriaBuilder().with("authorizedIds", "authorizeId", CriteriaBuilder.Builders.IN);
    }
}
