package com.flyfish.framework.service;

import com.flyfish.framework.domain.AdminUserDetails;
import com.flyfish.framework.domain.base.IUser;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.utils.CopyUtils;
import lombok.RequiredArgsConstructor;
import lombok.val;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.ListUtils;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import java.util.Collections;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class UserDetailsConverter {

    private final DepartmentService departmentService;

    /**
     * 将用户映射为用户详情，查询所有权限code
     *
     * @param user 用户信息
     * @return 结果
     */
    public Mono<UserDetails> mapToUserDetails(IUser user) {
        AdminUserDetails userDetail = new AdminUserDetails();
        CopyUtils.copyProps(user, userDetail);
        userDetail.setAuthorityCodes(Collections.singletonList(Department.PUBLIC));
        if (user.getType() == UserType.SUPER_ADMIN) {
            userDetail.setAuthorityCodes(null);
        }
        if (CollectionUtils.isNotEmpty(user.getDepartments())) {
            val departs = user.getDepartments().stream().map(Department::getId)
                    .collect(Collectors.toList());
            return departmentService.getSubCodes(departs)
                    .map(codes -> {
                        if (CollectionUtils.isNotEmpty(codes)) {
                            userDetail.setAuthorityCodes(ListUtils.union(codes, userDetail.getAuthorityCodes()));
                        }
                        return userDetail;
                    });
        }
        return Mono.just(userDetail);
    }
}
